@@ -1,27 +1,14 @@
 import sys
 import time
 
+from . import base_socket
 from . import exceptions
 from . import packet
 from . import payload
 
 
-class Socket(object):
+class Socket(base_socket.BaseSocket):
     """An Engine.IO socket."""
-    upgrade_protocols = ['websocket']
-
-    def __init__(self, server, sid):
-        self.server = server
-        self.sid = sid
-        self.queue = self.server.create_queue()
-        self.last_ping = None
-        self.connected = False
-        self.upgrading = False
-        self.upgraded = False
-        self.closing = False
-        self.closed = False
-        self.session = {}
-
     def poll(self):
         """Wait for packets to send to the client."""
         queue_empty = self.server.get_queue_empty_exception()
@@ -138,14 +125,14 @@ class Socket(object):
                 self.queue.join()
 
     def schedule_ping(self):
-        def send_ping():
-            self.last_ping = None
-            self.server.sleep(self.server.ping_interval)
-            if not self.closing and not self.closed:
-                self.last_ping = time.time()
-                self.send(packet.Packet(packet.PING))
+        self.server.start_background_task(self._send_ping)
 
-        self.server.start_background_task(send_ping)
+    def _send_ping(self):
+        self.last_ping = None
+        self.server.sleep(self.server.ping_interval)
+        if not self.closing and not self.closed:
+            self.last_ping = time.time()
+            self.send(packet.Packet(packet.PING))
 
     def _upgrade_websocket(self, environ, start_response):
         """Upgrade the connection from polling to websocket."""
@@ -154,11 +141,19 @@ class Socket(object):
         if self.server._async['websocket'] is None:
             # the selected async mode does not support websocket
             return self.server._bad_request()
-        ws = self.server._async['websocket'](self._websocket_handler)
-        return ws(environ, start_response)
+        ws = self.server._async['websocket'](
+            self._websocket_handler, self.server)
+        ws(environ, start_response)
+        return self.server._ok()
 
     def _websocket_handler(self, ws):
         """Engine.IO handler for websocket transport."""
+        def websocket_wait():
+            data = ws.wait()
+            if data and len(data) > self.server.max_http_buffer_size:
+                raise ValueError('packet is too large')
+            return data
+
         # try to set a socket timeout matching the configured ping interval
         # and timeout
         for attr in ['_sock', 'socket']:  # pragma: no cover
@@ -170,7 +165,7 @@ class Socket(object):
             # the socket was already connected, so this is an upgrade
             self.upgrading = True  # hold packet sends during the upgrade
 
-            pkt = ws.wait()
+            pkt = websocket_wait()
             decoded_pkt = packet.Packet(encoded_packet=pkt)
             if decoded_pkt.packet_type != packet.PING or \
                     decoded_pkt.data != 'probe':
@@ -181,7 +176,7 @@ class Socket(object):
             ws.send(packet.Packet(packet.PONG, data='probe').encode())
             self.queue.put(packet.Packet(packet.NOOP))  # end poll
 
-            pkt = ws.wait()
+            pkt = websocket_wait()
             decoded_pkt = packet.Packet(encoded_packet=pkt)
             if decoded_pkt.packet_type != packet.UPGRADE:
                 self.upgraded = False
@@ -213,6 +208,8 @@ class Socket(object):
                         ws.send(pkt.encode())
                 except:
                     break
+            ws.close()
+
         writer_task = self.server.start_background_task(writer)
 
         self.server.logger.info(
@@ -221,7 +218,7 @@ class Socket(object):
         while True:
             p = None
             try:
-                p = ws.wait()
+                p = websocket_wait()
             except Exception as e:
                 # if the socket is already closed, we can assume this is a
                 # downstream error of that
